;;; init-base.el --- Configuración base de emacs	-*- lexical-binding: t -*-

;; Author: kj <webmaster@outcontrol.net>
;; URL: https://git.kj2.me/kj/confi-emacs-actual

;;; Commentary:

;; Configuración general de Emacs.
;; No incluye paquetes/extensiones, solo configuraciones generales.

;;; Code:

;; Native compilation
(when (and (fboundp 'native-comp-available-p)
           (native-comp-available-p))
  (progn
    (setq native-comp-async-report-warnings-errors nil)
    (setq comp-deferred-compilation t)
    (setq package-native-compile t)
    ))

;; Instalar straight.el (reemplpazando package.el)
(defvar bootstrap-version)
(let ((bootstrap-file
      (expand-file-name "straight/repos/straight.el/bootstrap.el" user-emacs-directory))
      (bootstrap-version 5))
  (unless (file-exists-p bootstrap-file)
    (with-current-buffer
        (url-retrieve-synchronously
        "https://raw.githubusercontent.com/radian-software/straight.el/master/install.el"
        'silent 'inhibit-cookies)
      (goto-char (point-max))
      (eval-print-last-sexp)))
  (load bootstrap-file nil 'nomessage))

;; Instalar use-package si no está instalado
(straight-use-package 'use-package)
(setq use-package-always-ensure t)

;; Theme
(use-package dracula-theme
  :straight t
  :defer t
  :hook (after-init . (lambda()
                        (load-theme 'dracula t)
                        (set-face-attribute 'default nil :font "Fira Code Retina" :height 112) ; Font
                        ))
  )

(defconst private-dir  (expand-file-name "private" user-emacs-directory))
(defconst temp-dir (format "%s/cache" private-dir)
  "Hostname-based elisp temp directories.")

;; UTF-8 please
(set-charset-priority 'unicode)
(setq locale-coding-system   'utf-8)   ; pretty
(set-terminal-coding-system  'utf-8)   ; pretty
(set-keyboard-coding-system  'utf-8)   ; pretty
(set-selection-coding-system 'utf-8)   ; please
(prefer-coding-system        'utf-8)   ; with sugar on top
(setq default-process-coding-system '(utf-8-unix . utf-8-unix))

;; Emacs customizations
(scroll-bar-mode -1)                            ; Quitar scrollbar.
(tooltip-mode -1)                               ; Disable tooltips.
(global-display-line-numbers-mode)              ; Mostar número de línea.
(show-paren-mode 1)                             ; Habilitar resaltado de brackets.
(pending-delete-mode t)                         ; Cuando selecciono un texto y escribo, el texto se borra (emacs por defecto solo escribe donde está el cursor).
;; (global-hl-line-mode)                           ; Resaltar línea actual por defecto.

(setq-default
 cursor-type 'bar                               ; Usar la barrita como cursor
 cursor-in-non-selected-windows nil             ; Desaparecer el cursor en frames no activos.
 indent-tabs-mode nil                           ; Cambiar tabs por espacios.
 inhibit-startup-message t                      ; Eliminar el mensaje de inicio.
 )
(setq
 custom-file (concat private-dir "/.custom.el") ; Cabiar la ruta del código que se genera al isntalar un theme o package
 ac-ignore-case nil                             ; Desactivar el autocapitalizado.
 ac-disable-faces nil                           ; Auto-complete hablitado incluso entre comillas (fuente: https://bit.ly/3a9wCB4).
 tags-revert-without-query 1                    ; Recargar tags (ctags) sin pedir confirmación.
 )
;; Leve transparencia en emacs por defecto.
(set-frame-parameter nil 'alpha-background 99)

;; Backups enabled, use nil to disable
(setq
 history-length                     1000
 backup-inhibited                   nil
 make-backup-files                  nil
 auto-save-default                  nil
 auto-save-list-file-name           (concat temp-dir "/autosave")
 create-lockfiles                   nil
 backup-directory-alist            `((".*" . ,(concat temp-dir "/backup/")))
 auto-save-file-name-transforms    `((".*" ,(concat temp-dir "/auto-save-list/") t)))

;; Configuración cuando es un server
(defun setup-daemon ()
  "Carga la configuración del modo daemon."
  (message "Corriendo en modo daemon.")
  (set-face-attribute 'default nil :font "Fira Code Retina" :height 112)

  ;; Abrir primero el dashboard
  (setq initial-buffer-choice (lambda ()
                                (get-buffer "*dashboard*")
                                (dashboard-refresh-buffer)))

  ;; Cerrar buffers al cerrar emacsclient
  (add-hook 'delete-frame-functions
            (lambda (frame)
              (mapc 'kill-buffer (delq (get-buffer "*dashboard*") (buffer-list)))
              ))
  )

(if (daemonp)
    (add-hook 'after-make-frame-functions
              (lambda (frame)
                (with-selected-frame frame
                  (setup-daemon))))
  (message "Corriendo en modo normal."))

;(add-hook 'window-setup-hook 'toggle-frame-maximized t)

(load custom-file)

(provide 'init-base)
;;; init-base.el ends here
